%% Main script for stochastic denoising. Algorithm described in
%% F. Estrada, D. Fleet, and A. Jepson, "Stochastic Image Denoising",
%% BMVC 2009.
%%
%% Edit this file to select the apropriate executable format
%% for your local installation:
%%
%% Linux 32bit, Linux 64bit, and Win32 versions are provided,
%% the fast version should work well most of the time, however
%% it trades speed for increased memory usage. If you get
%% out-of-memory errors, or your system starts paging to
%% virtual storage, switch to the slower, but more memory
%% efficient version.
%%
%% function [imD1]=stochastic_denoise(im,sigma,thresh,npaths,NN)
%%
%% Input:
%%
%% im - An input RGB image (for grayscale images, replicate
%%      the brightness plane)
%% sigma - Standard deviation of the noise process (less than 1)
%%         the larger the value, the stronger the denoising,
%%         but also the more likely that texture and soft
%%         boundaries will be blurred.
%% thresh - Stopping threshold for random walks. Values from
%%          .000001 to .001 yield good results, the smaller
%%          the value, the longer the random walks (and 
%%          usually this yields better results). However
%%          using smaller values increases runtime.
%% npaths - Number of trials per pixel, the more trials the
%%          more accurate the final estimate of the pixel
%%          colour. Values as small as 10 yield stable
%%          estimates, while values larger than 50 don't
%%          generally produce further improvement.
%%          Runtime increases linearly with this value.
%% NN - An integer used to distinguish the temp. files
%%      created by this script. Useful when running several
%%      instances on the same machine, give each instance
%%      a different value, otherwise they will overwrite
%%      each other's temp. files.
%%
%% Return values:
%%
%% imD1 - A denoised RGB image with values in [0 1]
%%
%% (c) 2009 F. Estrada, D. Fleet, and A. Jepson
%% For questions or bug reports, please contact
%% festrada [you know what] utsc [period] utoronto [period] ca
%%

function [imD1]=stochastic_denoise(im,sigma,thresh,npaths,NN)

tic;

%% SELECT THE APPROPRIATE EXECUTABLE FROM THE LIST BELOW:
%exec_name='stochastic_denoise_win32.exe';       % Slow method, win32
%exec_name='stochastic_denoise_fast_win32.exe';  % Fast method, win32
%exec_name='stochastic_denoise_i386';       % Slow method, Linux 32bit
%exec_name='stochastic_denoise_fast_i386';  % Fast method, Linux 32bit
%exec_name='stochastic_denoise_ia64';	    % Slow method, Linux 64bit
exec_name='stochastic_denoise_fast_ia64';  % Fast method, Linux 64bit

% Make double and normalize to 0-1 if necessary
if (~isfloat(im)) im=double(im); end;
if (max(max(max(im)))>1.1) im=im/255; end;
if (size(im,3)==1) im=repmat(im,[1 1 3]); end;

% Output a binary file for the executable module
imT(:,:,1)=im(:,:,1)';
imT(:,:,2)=im(:,:,2)';
imT(:,:,3)=im(:,:,3)';
imT=reshape(imT,[size(imT,1)*size(imT,2) 3])';
st=sprintf('temp_img_%d.dat',NN);
f=fopen(st,'w');
fwrite(f,imT,'float');
fclose(f);

% Standard denoising procedure: Isotropic random walks

com=sprintf('./%s temp_img_%d.dat %d %d %g %g %d 4 temp_denoised_%d.dat',...
             exec_name,NN,size(im,2),size(im,1),sigma,thresh,npaths,NN)

unix(com);

st=sprintf('temp_denoised_%d.dat',NN);
f=fopen(st);
imDt=fread(f,[3 inf],'float')';
fclose(f);

% Now re-arrange the elements in imD
imDt=reshape(imDt,[size(im,2) size(im,1) 3]);
imD1(:,:,1)=imDt(:,:,1)';
imD1(:,:,2)=imDt(:,:,2)';
imD1(:,:,3)=imDt(:,:,3)';

% Uncomment the lines below to display the original and denoised images
%figure(1);clf;;image(im);axis image;axis off;
%figure(2);image(imD1);axis image;axis off;

fprintf(2,'This call to stochastic_denoise.m took:\n');
toc;

return;
